@@ -10,11 +10,31 @@
 //!
 //! Functions are organized into several modules according to the services
 //! provided by them.
+//!
+//! # Overview
+//!
+//! As far as we expect, input method (in fact, the way for text input and
+//! output, especially in multi-lingual environments) implementations are
+//! becoming more and more complex in system integrations.  The emerging
+//! standard for practical and flexible development cycles is required, so that
+//! we are facing the impacts from various Chinese input method implementations
+//! and the integration into existing framework or system designs.  At the
+//! result, Chewing Input Method (abbreviated as "Chewing") attempts to be one
+//! of the approaches to solve and ease the problems with the inclusion of base
+//! classes, on the basis of cross-platform, and cross-operating-environment
+//! derivative classes, which are built as the abstract backbone of intelligent
+//! Chinese phonetic input method implementations.  From the perspectives,
+//! Chewing defines the abstract behavior how an intelligent phonetic IM should
+//! works via the common interface, and Chewing permits the extra parameters and
+//! properties for the input method implementations to extend their specific
+//! functionality.
 
 mod io;
 mod logger;
 mod public;
 
+pub mod version;
+
 /// Initializes chewing context and environment settings.
 ///
 /// Most of the Chewing IM APIs require a
@@ -237,6 +257,14 @@ pub mod input {
     /// The value of key should be in the range between ASCII character code
     /// from 0 to 9.
     pub use super::io::chewing_handle_Numlock;
+
+    pub use super::public::KEYSTROKE_IGNORE;
+
+    pub use super::public::KEYSTROKE_COMMIT;
+
+    pub use super::public::KEYSTROKE_BELL;
+
+    pub use super::public::KEYSTROKE_ABSORB;
 }
 
 /// Keyboard layout and variants setting.
@@ -290,6 +318,9 @@ pub mod layout {
     /// * KB_THL_PINYIN
     /// * KB_MPS2_PINYIN
     /// * KB_CARPALX
+    /// * KB_COLEMAK_DH_ANSI
+    /// * KB_COLEMAK_DH_ORTH
+    /// * KB_WORKMAN
     ///
     /// See also [chewing_kbtype_Enumerate] for getting the list of supported
     /// layouts programmatically.
@@ -793,6 +824,17 @@ pub mod output {
     /// This function fails if the IM editor is not in entering state.
     pub use super::io::chewing_clean_bopomofo_buf;
 
+    /// Acknowledge the commit buffer and aux output buffer.
+    ///
+    /// Chewing automatically acknowledges and clear the output buffers after
+    /// processing new input events.
+    ///
+    /// After handling the ephemeral output buffer like the commit buffer and
+    /// the aux output buffer, IM wrappers can proactively acknowledge and clear
+    /// the buffers. This can be used so that IM wrappers don't have to remember
+    /// whether an output has been handled or not.
+    pub use super::io::chewing_ack;
+
     pub use super::public::IntervalType;
 }
 
@@ -946,6 +988,14 @@ pub mod globals {
     /// Returns whether the automatic learning is enabled or disabled.
     pub use super::io::chewing_get_autoLearn;
 
+    pub use super::io::chewing_config_has_option;
+
+    pub use super::io::chewing_config_get_int;
+    pub use super::io::chewing_config_set_int;
+
+    pub use super::io::chewing_config_get_str;
+    pub use super::io::chewing_config_set_str;
+
     pub use super::public::MAX_CHI_SYMBOL_LEN;
     pub use super::public::MIN_CHI_SYMBOL_LEN;
 
