@@ -2,7 +2,7 @@ use std::cmp::min;
 
 use crate::{
     conversion::{Composition, Gap, Interval},
-    dictionary::{Dictionary, Layered},
+    dictionary::{Dictionary, Layered, LookupStrategy},
     editor::{EditorError, SharedState},
 };
 
@@ -12,16 +12,22 @@ pub(crate) struct PhraseSelector {
     end: usize,
     forward_select: bool,
     orig: usize,
+    lookup_strategy: LookupStrategy,
     com: Composition,
 }
 
 impl PhraseSelector {
-    pub(crate) fn new(forward_select: bool, com: Composition) -> PhraseSelector {
+    pub(crate) fn new(
+        forward_select: bool,
+        lookup_strategy: LookupStrategy,
+        com: Composition,
+    ) -> PhraseSelector {
         PhraseSelector {
             begin: 0,
             end: com.len(),
             forward_select,
             orig: 0,
+            lookup_strategy,
             com,
         }
     }
@@ -45,7 +51,10 @@ impl PhraseSelector {
                 !syllables.is_empty(),
                 "should not enter here if there's no syllable in range"
             );
-            if dict.lookup_first_phrase(&syllables).is_some() {
+            if dict
+                .lookup_first_phrase(&syllables, self.lookup_strategy)
+                .is_some()
+            {
                 break;
             }
             if self.forward_select {
@@ -56,6 +65,12 @@ impl PhraseSelector {
         }
     }
 
+    pub(crate) fn init_single_word(&mut self, cursor: usize) {
+        self.orig = cursor;
+        self.end = min(cursor, self.com.len());
+        self.begin = self.end - 1;
+    }
+
     pub(crate) fn begin(&self) -> usize {
         self.begin
     }
@@ -75,7 +90,10 @@ impl PhraseSelector {
                 }
             }
             let syllables = &self.com.symbols()[begin..end];
-            if dict.lookup_first_phrase(&syllables).is_some() {
+            if dict
+                .lookup_first_phrase(&syllables, self.lookup_strategy)
+                .is_some()
+            {
                 return Some((begin, end));
             }
         }
@@ -101,7 +119,10 @@ impl PhraseSelector {
                 }
             }
             let syllables = &self.com.symbols()[begin..end];
-            if dict.lookup_first_phrase(&syllables).is_some() {
+            if dict
+                .lookup_first_phrase(&syllables, self.lookup_strategy)
+                .is_some()
+            {
                 return Some((begin, end));
             }
         }
@@ -149,11 +170,15 @@ impl PhraseSelector {
             } else {
                 self.begin += 1;
                 if self.begin == self.end {
+                    self.begin -= 1;
                     self.begin = self.after_previous_break_point(self.begin);
                 }
             }
             let syllables = &self.com.symbols()[self.begin..self.end];
-            if dict.lookup_first_phrase(&syllables).is_some() {
+            if dict
+                .lookup_first_phrase(&syllables, self.lookup_strategy)
+                .is_some()
+            {
                 break;
             }
         }
@@ -198,7 +223,10 @@ impl PhraseSelector {
 
     pub(crate) fn candidates(&self, editor: &SharedState, dict: &Layered) -> Vec<String> {
         let mut candidates = dict
-            .lookup_all_phrases(&&self.com.symbols()[self.begin..self.end])
+            .lookup_all_phrases(
+                &&self.com.symbols()[self.begin..self.end],
+                self.lookup_strategy,
+            )
             .into_iter()
             .map(|phrase| phrase.into())
             .collect::<Vec<String>>();
@@ -208,7 +236,7 @@ impl PhraseSelector {
                 .alt_syllables(self.com.symbol(self.begin).unwrap().to_syllable().unwrap());
             for &syl in alt {
                 candidates.extend(
-                    dict.lookup_all_phrases(&[syl])
+                    dict.lookup_all_phrases(&[syl], self.lookup_strategy)
                         .into_iter()
                         .map(|ph| ph.into()),
                 )
@@ -231,7 +259,7 @@ impl PhraseSelector {
 mod tests {
     use crate::{
         conversion::{Composition, Interval, Symbol},
-        dictionary::TrieBuf,
+        dictionary::{LookupStrategy, TrieBuf},
         syl,
         zhuyin::Bopomofo::*,
     };
@@ -247,6 +275,7 @@ mod tests {
             end: 1,
             forward_select: false,
             orig: 0,
+            lookup_strategy: LookupStrategy::Standard,
             com,
         };
         let dict = TrieBuf::from([(vec![syl![C, E, TONE4]], vec![("測", 100)])]);
@@ -266,6 +295,7 @@ mod tests {
             end: 1,
             forward_select: false,
             orig: 0,
+            lookup_strategy: LookupStrategy::Standard,
             com,
         };
         let dict = TrieBuf::from([(vec![syl![C, E, TONE4]], vec![("測", 100)])]);
@@ -281,6 +311,7 @@ mod tests {
             end: 1,
             forward_select: true,
             orig: 0,
+            lookup_strategy: LookupStrategy::Standard,
             com,
         };
         let dict = TrieBuf::from([(vec![syl![C, E, TONE4]], vec![("測", 100)])]);
@@ -300,6 +331,7 @@ mod tests {
             end: 1,
             forward_select: true,
             orig: 0,
+            lookup_strategy: LookupStrategy::Standard,
             com,
         };
         let dict = TrieBuf::from([(vec![syl![C, E, TONE4]], vec![("測", 100)])]);
@@ -320,6 +352,7 @@ mod tests {
             end: 2,
             forward_select: false,
             orig: 0,
+            lookup_strategy: LookupStrategy::Standard,
             com,
         };
 
@@ -339,6 +372,7 @@ mod tests {
             end: 2,
             forward_select: false,
             orig: 0,
+            lookup_strategy: LookupStrategy::Standard,
             com,
         };
 
@@ -367,6 +401,7 @@ mod tests {
             end: 2,
             forward_select: false,
             orig: 0,
+            lookup_strategy: LookupStrategy::Standard,
             com,
         };
 
