@@ -10,7 +10,7 @@ use super::{KeyBehavior, SyllableEditor};
 const MAX_PINYIN_LEN: usize = 10;
 
 /// TODO: docs
-#[derive(Debug, Default)]
+#[derive(Debug, Default, Clone, Copy, PartialEq, Eq)]
 pub enum PinyinVariant {
     /// TODO: docs
     #[default]
@@ -22,7 +22,7 @@ pub enum PinyinVariant {
 }
 
 /// TODO: docs
-#[derive(Default, Debug)]
+#[derive(Default, Debug, Clone)]
 pub struct Pinyin {
     key_seq: String,
     syllable: Syllable,
@@ -71,6 +71,9 @@ impl Pinyin {
 
 impl SyllableEditor for Pinyin {
     fn key_press(&mut self, key: KeyEvent) -> KeyBehavior {
+        if self.key_seq.is_empty() && !key.code.is_atoz() {
+            return KeyBehavior::KeyError;
+        }
         if ![
             KeyCode::Space,
             KeyCode::N1,
@@ -85,36 +88,10 @@ impl SyllableEditor for Pinyin {
                 // buffer is full, ignore this keystroke
                 return KeyBehavior::NoWord;
             }
-            let ch = match key.code {
-                KeyCode::A => 'a',
-                KeyCode::B => 'b',
-                KeyCode::C => 'c',
-                KeyCode::D => 'd',
-                KeyCode::E => 'e',
-                KeyCode::F => 'f',
-                KeyCode::G => 'g',
-                KeyCode::H => 'h',
-                KeyCode::I => 'i',
-                KeyCode::J => 'j',
-                KeyCode::K => 'k',
-                KeyCode::L => 'l',
-                KeyCode::M => 'm',
-                KeyCode::N => 'n',
-                KeyCode::O => 'o',
-                KeyCode::P => 'p',
-                KeyCode::Q => 'q',
-                KeyCode::R => 'r',
-                KeyCode::S => 's',
-                KeyCode::T => 't',
-                KeyCode::U => 'u',
-                KeyCode::V => 'v',
-                KeyCode::W => 'w',
-                KeyCode::X => 'x',
-                KeyCode::Y => 'y',
-                KeyCode::Z => 'z',
-                _ => return KeyBehavior::KeyError,
-            };
-            self.key_seq.push(ch);
+            if !key.unicode.is_ascii_alphabetic() {
+                return KeyBehavior::KeyError;
+            }
+            self.key_seq.push(key.unicode);
             return KeyBehavior::Absorb;
         }
 
@@ -180,61 +157,98 @@ impl SyllableEditor for Pinyin {
         let mut medial = fina.and_then(|f| f.medial);
         let mut rime = fina.and_then(|f| f.rime);
 
-        if let Some(Bopomofo::I) = rime {
-            match initial {
-                Some(Bopomofo::ZH) | Some(Bopomofo::CH) | Some(Bopomofo::SH)
-                | Some(Bopomofo::R) | Some(Bopomofo::Z) | Some(Bopomofo::C) | Some(Bopomofo::S) => {
-                    medial.take();
-                    rime.take();
+        /* Hanyu empty rime
+         * ㄓ/ㄔ/ㄕ/ㄖ/ㄗ/ㄘ/ㄙ + -i, -i is empty rime, not ㄧ
+         * */
+        match self.variant {
+            PinyinVariant::HanyuPinyin => {
+                if matches!(
+                    (medial, rime),
+                    (Some(Bopomofo::I), None) | (None, Some(Bopomofo::I))
+                ) {
+                    match initial {
+                        Some(Bopomofo::ZH) | Some(Bopomofo::CH) | Some(Bopomofo::SH)
+                        | Some(Bopomofo::R) | Some(Bopomofo::Z) | Some(Bopomofo::C)
+                        | Some(Bopomofo::S) => {
+                            medial.take();
+                            rime.take();
+                        }
+                        _ => (),
+                    }
+                }
+            }
+            _ => {}
+        }
+
+        /* Hanyu uan/un/u :
+         * ㄐ/ㄑ/ㄒ + -uan, -uan is ㄩㄢ, not ㄨㄢ
+         * ㄐ/ㄑ/ㄒ + -un,  -un is ㄩㄣ, not ㄨㄣ
+         * ㄐ/ㄑ/ㄒ + -u,   -u is ㄧ, not ㄨ
+         */
+        match self.variant {
+            PinyinVariant::HanyuPinyin => match initial {
+                Some(Bopomofo::J) | Some(Bopomofo::Q) | Some(Bopomofo::X) => {
+                    match (medial, rime) {
+                        (Some(Bopomofo::U), Some(Bopomofo::AN))
+                        | (Some(Bopomofo::U), Some(Bopomofo::EN))
+                        | (Some(Bopomofo::U), None) => {
+                            medial.replace(Bopomofo::IU);
+                        }
+                        _ => (),
+                    };
                 }
                 _ => (),
-            }
+            },
+            _ => {}
         }
 
-        match initial {
-            Some(Bopomofo::J) | Some(Bopomofo::Q) | Some(Bopomofo::X) => {
-                match (medial, rime) {
-                    (Some(Bopomofo::U), Some(Bopomofo::AN))
-                    | (Some(Bopomofo::U), Some(Bopomofo::EN))
-                    | (Some(Bopomofo::U), None) => {
-                        medial.replace(Bopomofo::IU);
-                    }
-                    _ => (),
-                };
-            }
-            _ => (),
-        }
-
-        match medial {
-            Some(Bopomofo::I) | Some(Bopomofo::IU) => {
-                match initial {
-                    Some(Bopomofo::S) | Some(Bopomofo::SH) => {
-                        initial.replace(Bopomofo::X);
-                    }
-                    Some(Bopomofo::C) | Some(Bopomofo::CH) => {
-                        initial.replace(Bopomofo::Q);
-                    }
-                    _ => (),
-                };
-            }
-            _ => {
-                if initial == Some(Bopomofo::J) {
-                    initial.replace(Bopomofo::ZH);
+        /* THL/MPS2 s/sh/c/ch/j :
+         * s-  + ー/ㄩ, s-  is ㄒ, not ㄙ (THL/Tongyong)
+         * sh- + ー/ㄩ, sh- is ㄒ, not ㄕ (MPS2)
+         * c-  + ー/ㄩ, c-  is ㄑ, not ㄘ (Tongyong)
+         * ch- + ㄧ/ㄩ, ch- is ㄑ, not ㄔ (THL)
+         * j-  + other than ー/ㄩ, j-  is ㄓ, not ㄐ (MPS2)
+         */
+        match self.variant {
+            PinyinVariant::ThlPinyin | PinyinVariant::Mps2Pinyin => match medial {
+                Some(Bopomofo::I) | Some(Bopomofo::IU) => {
+                    match initial {
+                        Some(Bopomofo::S) | Some(Bopomofo::SH) => {
+                            initial.replace(Bopomofo::X);
+                        }
+                        Some(Bopomofo::C) | Some(Bopomofo::CH) => {
+                            initial.replace(Bopomofo::Q);
+                        }
+                        _ => (),
+                    };
                 }
-            }
+                _ => {
+                    if initial == Some(Bopomofo::J) {
+                        initial.replace(Bopomofo::ZH);
+                    }
+                }
+            },
+            PinyinVariant::HanyuPinyin => {}
         }
 
-        match initial {
-            Some(Bopomofo::B) | Some(Bopomofo::P) | Some(Bopomofo::M) | Some(Bopomofo::F) => {
-                match (medial, rime) {
-                    (Some(Bopomofo::U), Some(Bopomofo::ENG))
-                    | (Some(Bopomofo::U), Some(Bopomofo::O)) => {
-                        medial.take();
-                    }
-                    _ => (),
-                };
-            }
-            _ => (),
+        /* THL supplemental set
+         * ㄅ/ㄆ/ㄇ/ㄈ + -ㄨㄥ, -ㄨㄥ is another reading of -ㄥ
+         * ㄅ/ㄆ/ㄇ/ㄈ + -ㄨㄛ, -ㄨㄛ is another reading of -ㄛ
+         */
+        match self.variant {
+            PinyinVariant::ThlPinyin | PinyinVariant::Mps2Pinyin => match initial {
+                Some(Bopomofo::B) | Some(Bopomofo::P) | Some(Bopomofo::M) | Some(Bopomofo::F) => {
+                    match (medial, rime) {
+                        (Some(Bopomofo::U), Some(Bopomofo::ENG))
+                        | (Some(Bopomofo::U), Some(Bopomofo::O)) => {
+                            medial.take();
+                        }
+                        _ => (),
+                    };
+                }
+                _ => (),
+            },
+            _ => {}
         }
 
         self.key_seq.clear();
@@ -256,6 +270,10 @@ impl SyllableEditor for Pinyin {
         KeyBehavior::Commit
     }
 
+    fn fuzzy_key_press(&mut self, key: KeyEvent) -> KeyBehavior {
+        self.key_press(key)
+    }
+
     fn is_empty(&self) -> bool {
         self.key_seq.is_empty()
     }
@@ -277,6 +295,10 @@ impl SyllableEditor for Pinyin {
     fn key_seq(&self) -> Option<String> {
         Some(self.key_seq.clone())
     }
+
+    fn clone(&self) -> Box<dyn SyllableEditor> {
+        Box::new(Clone::clone(self))
+    }
 }
 
 struct AmbiguousMapEntry {
@@ -505,3 +527,71 @@ mod table {
         fin!("z", None, None),
     ];
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::{
+        editor::{
+            keyboard::{AnyKeyboardLayout, KeyCode, KeyboardLayout},
+            zhuyin_layout::SyllableEditor,
+        },
+        syl,
+        zhuyin::Bopomofo,
+    };
+
+    use super::Pinyin;
+
+    #[test]
+    fn hanyu_empty_rime_zi() {
+        let keyboard = AnyKeyboardLayout::qwerty();
+        let mut hanyu = Pinyin::hanyu();
+
+        hanyu.key_press(keyboard.map(KeyCode::Z));
+        hanyu.key_press(keyboard.map(KeyCode::I));
+        hanyu.key_press(keyboard.map(KeyCode::N1));
+
+        assert_eq!(syl![Bopomofo::Z], hanyu.read());
+    }
+
+    #[test]
+    fn hanyu_empty_rime_zhi() {
+        let keyboard = AnyKeyboardLayout::qwerty();
+        let mut hanyu = Pinyin::hanyu();
+
+        hanyu.key_press(keyboard.map(KeyCode::Z));
+        hanyu.key_press(keyboard.map(KeyCode::H));
+        hanyu.key_press(keyboard.map(KeyCode::I));
+        hanyu.key_press(keyboard.map(KeyCode::N1));
+
+        assert_eq!(syl![Bopomofo::ZH], hanyu.read());
+    }
+
+    #[test]
+    fn hanyu_uan_un_u() {
+        let keyboard = AnyKeyboardLayout::qwerty();
+        let mut hanyu = Pinyin::hanyu();
+
+        hanyu.key_press(keyboard.map(KeyCode::J));
+        hanyu.key_press(keyboard.map(KeyCode::U));
+        hanyu.key_press(keyboard.map(KeyCode::A));
+        hanyu.key_press(keyboard.map(KeyCode::N));
+        hanyu.key_press(keyboard.map(KeyCode::N1));
+
+        assert_eq!(syl![Bopomofo::J, Bopomofo::IU, Bopomofo::AN], hanyu.read());
+
+        hanyu.clear();
+        hanyu.key_press(keyboard.map(KeyCode::Q));
+        hanyu.key_press(keyboard.map(KeyCode::U));
+        hanyu.key_press(keyboard.map(KeyCode::N));
+        hanyu.key_press(keyboard.map(KeyCode::N1));
+
+        assert_eq!(syl![Bopomofo::Q, Bopomofo::IU, Bopomofo::EN], hanyu.read());
+
+        hanyu.clear();
+        hanyu.key_press(keyboard.map(KeyCode::X));
+        hanyu.key_press(keyboard.map(KeyCode::U));
+        hanyu.key_press(keyboard.map(KeyCode::N1));
+
+        assert_eq!(syl![Bopomofo::X, Bopomofo::IU], hanyu.read());
+    }
+}
